from cloud_storages.backends.dropbox import *
from django.test import *
from django.core.files.base import ContentFile

class DropBoxStorageTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        # print("setUpTestData: Run once to set up non-modified data for all class methods.")
        pass
    
    def setUp(self, *args):
        # print("setUp: Run once for every test method to setup clean data.")
        self.storage = DropBoxStorage()
        # self.file_name = "documents/resume/UGC-NET-220520144573.pdf"
        self.file_name = "images/dp/Screenshot 2023-06-03 210915.png"

    def tearDown(self, *args):
        # Clean up run after every test method.
        pass

    # def test_save(self, *args):
    #     c = ContentFile(b'')
    #     self.storage.save(name=self.file_name, content=c)

    # def test_open(self, *args):
    #     self.storage.open(name=self.file_name)

    # def test_save(self, *args):
    #     test_content = ContentFile(content="Test File")
    #     self.storage.save(name=self.file_name, content=test_content)
    
    # def test_delete(self, *args):
    #     self.storage.delete(name=self.file_name)


from django.conf import settings

def setting(name, default=None):
    """
    Helper function to get a Django setting by name. If setting doesn't exists
    it will return a default.

    :param name: Name of setting
    :type name: str
    :param default: Value if setting is not found
    :returns: Setting's value
    """
    return getattr(settings, name, default)


def dropBoxErrorMsg(err_tag):
    err_dict = {
        "not_found": "not_found: There is nothing at the given path.",
        "not_file": "not_file: We were expecting a file, but the given path refers to something that isn't a file.",
        "not_folder": "not_folder: We were expecting a folder, but the given path refers to something that isn't a folder.",
        "restricted_content": "restricted_content: The file cannot be proceed because the content is restricted."
    }
    return err_dict[err_tag]

class FileNameLengthError(Exception):
    """Exception raised for errors when file name length is too large or too small.
    """
    def __init__(self, message, min_length=None, max_length=None):
        self.min_length = min_length
        self.max_length = max_length
        self.message = message
        super().__init__(self.message)
    def __str__(self):
        return self.message

class ContentDoesNotExistsError(Exception):
    """Exception raised for errors when the requested file or folder does not exists.
    """
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)
    def __str__(self):
        return self.message


import requests
import os
import traceback
from io import BytesIO
import inspect

from django.core.files.base import ContentFile
from django.core.files.storage import Storage
from django.core.files import File
from django.utils.deconstruct import deconstructible
from django.utils._os import safe_join

import dropbox
from dropbox import sharing as dbx_sharing
from dropbox.exceptions import ApiError
from dropbox.files import *

from cloud_storages.utils import *

_DEFAULT_TIMEOUT = 100
_DEFAULT_MODE = 'add'

@deconstructible
class DropBoxStorage(Storage):
    CHUNK_SIZE = 4 * 1024 * 1024
    DROPBOX_PERMANENT_LINK = False
    def __init__(self):
        self.OVERWRITE_FILE = setting('OVERWRITE_FILE', False)
        self.CLOUD_STORAGE_CREATE_NEW_IF_SAME_CONTENT = setting('CLOUD_STORAGE_CREATE_NEW_IF_SAME_CONTENT', True)
        self.DROPBOX_OAUTH2_ACCESS_TOKEN = setting('DROPBOX_OAUTH2_ACCESS_TOKEN')
        self.DROPBOX_OAUTH2_REFRESH_TOKEN = setting('DROPBOX_OAUTH2_REFRESH_TOKEN')
        self.DROPBOX_APP_KEY = setting('DROPBOX_APP_KEY')
        self.DROPBOX_APP_SECRET = setting('DROPBOX_APP_SECRET')
        self.DROPBOX_PERMANENT_LINK = DROPBOX_PERMANENT_LINK = setting('DROPBOX_PERMANENT_LINK', False)
        self.DROPBOX_ROOT_PATH = setting('DROPBOX_ROOT_PATH')
        self.MEDIA_URL = setting('MEDIA_URL')
        self.timeout = setting('DROPBOX_TIMEOUT', _DEFAULT_TIMEOUT)
        self.write_mode = setting('DROPBOX_WRITE_MODE', _DEFAULT_MODE)
        self.dbx = dropbox.Dropbox(app_key=self.DROPBOX_APP_KEY, app_secret=self.DROPBOX_APP_SECRET, oauth2_refresh_token=self.DROPBOX_OAUTH2_REFRESH_TOKEN)
        self.dbx.users_get_current_account()

    def _full_path(self, name):
        if name == '/':
            name = ''
        if name[0] in ["/", "\\"]:
            name = name[1:]
        joined_path = os.path.join(self.DROPBOX_ROOT_PATH, name).replace("\\", "/")
        return joined_path

    def open(self, name, mode="rb"):
        """Retrieve the specified file from storage."""
        return self._open(name, mode)
    def _open(self, name, mode='rb'):
        full_name = self._full_path(name)
        try:
            file_metadata, response = self.dbx.files_download(full_name)
            file_content = BytesIO(response.content)
            response_file = File(file_content, name=name)
            return response_file
        except ApiError as e:
            err = e.error
            if err.is_path_lookup():
                lookUpError = err.get_path_lookup()
                error_msg = dropBoxErrorMsg(lookUpError._tag)
                raise ContentDoesNotExistsError(error_msg)
            raise e
           
        
    def save(self, name, content, max_length=None):
        """
        Save new content to the file specified by name. The content should be
        a proper File object or any Python file-like object, ready to be read
        from the beginning.
        """
        # Get the proper name for the file, as it will actually be saved.
        if name is None:
            name = content.name
        # if not hasattr(content, "chunks"):
        #     content = File(content, name)
        path = self.get_available_name(name, content, max_length=max_length)
        if path[1] is None:
            self._save(path[0], content)
        return path[0]
    def _save(self, name, content):
        full_name = self._full_path(name)
        content.open()
        if content.size <= self.CHUNK_SIZE:
            self.dbx.files_upload(content.read(), full_name, mode=WriteMode(self.write_mode))
        else:
            self._chunked_upload(content, full_name)
        content.close()
        return name

    def get_available_name(self, name, content, max_length=None):
        """
        Return a filename that's free on the target storage system and
        available for new content to be written to.
        """
        formatted_name = self.get_valid_name(name)
        new_name = formatted_name
        if self.OVERWRITE_FILE:
            try:
                self.delete(new_name)
            except ContentDoesNotExistsError:
                pass
        else:
            index = 0
            while(1):
                index += 1
                if self.exists(new_name): # check file existence with file name
                    if not self.CLOUD_STORAGE_CREATE_NEW_IF_SAME_CONTENT:
                        # check file existence with file's contents
                        remote_file = self.open(name=new_name)
                        remote_file_data = remote_file.read()
                        content_data = content.read()
                        if remote_file_data == content_data:
                            return (new_name, 'Exists')
                    new_name = self.get_alternative_name(formatted_name, index=index)
                    continue
                break
        return (new_name, None)
    
    def generate_filename(self, filename):
        """
        Validate the filename by calling get_valid_name() and return a filename
        to be passed to the save() method.
        """
        name = self.get_valid_name(filename)
        return name

    def get_valid_name(self, name):
        """
        Return a filename, based on the provided filename, that's suitable for
        use in the target storage system.
        """
        name = str(name).replace("\\", "/")
        # name = self._full_path(name)
        return name
    
    def get_alternative_name(self, file_root, file_ext=None, index=0):
        """
        Return an alternative filename if one exists to the filename.
        """
        res = file_root.rsplit('.', 1)  # Split on last occurrence of delimiter
        file_name = f"{res[0]}({index})"
        file_ext = res[1]
        updated_name =  f"{file_name}.{file_ext}"
        # updated_name = self._full_path(updated_name)
        return updated_name

    def delete(self, name):
        """
        Delete the specified file from the storage system.
        """
        full_name = self._full_path(name)
        try:
            self.dbx.files_delete_v2(full_name)
        except ApiError as e:
            err = e.error
            if err.is_path_lookup():
                lookUpError = err.get_path_lookup()
                error_msg = dropBoxErrorMsg(lookUpError._tag)
                raise ContentDoesNotExistsError(error_msg)
            raise e
                
    def exists(self, name):
        """
        Return True if a file referenced by the given name already exists in the
        storage system, or False if the name is available for a new file.
        """
        full_name = self._full_path(name)
        try:
            return bool(self.dbx.files_get_metadata(full_name))
        except ApiError as e:
            err = e.error
            if err.is_path_lookup():
                # lookUpError = err.get_path_lookup()
                # error_msg = dropBoxErrorMsg(lookUpError._tag)
                return False
            raise e
    
    def listdir(self, path):
        """
        List the contents of the specified path. Return a 2-tuple of lists:
        the first item being directories, the second item being files.
        """
        directories, files = [], []
        metadata = self.dbx.files_list_folder(path)
        for entry in metadata.entries:
            if isinstance(entry, FolderMetadata):
                directories.append(entry.name)
            else:
                files.append(entry.name)
        return directories, files
    
    def size(self, name):
        """
        Return the total size, in bytes, of the file specified by name.
        """
        full_name = self._full_path(name)
        try:
            metadata = self.dbx.files_get_metadata(full_name)
            return metadata.size
        except ApiError as e:
            err = e.error
            if err.is_path_lookup():
                lookUpError = err.get_path_lookup()
                error_msg = dropBoxErrorMsg(lookUpError._tag)
                raise ContentDoesNotExistsError(error_msg)
            raise e

    def url(self, name, permanent_link=DROPBOX_PERMANENT_LINK):
        """
        Return an absolute URL where the file's contents can be accessed directly by a web browser.
        """
        full_name = self._full_path(name)
        try:
            if not permanent_link:
                media = self.dbx.files_get_temporary_link(full_name)
                file_url = media.link
            else:
                dbx_share_settings = dbx_sharing.SharedLinkSettings(allow_download=True)
                media = self.dbx.sharing_create_shared_link_with_settings(full_name, settings=dbx_share_settings)
                file_url = str(media.url)[:-1]+"1"
            return file_url
        except Exception:
            return None

    def get_accessed_time(self, name):
        """
        Return the last accessed time (as a datetime) of the file specified by name.
        The datetime will be timezone-aware if USE_TZ=True.
        """
        full_name = self._full_path(name)
        try:
            last_accessed = self.dbx.files_get_metadata(full_name).client_modified
            return last_accessed
        except ApiError as e:
            err = e.error
            if err.is_path_lookup():
                lookUpError = err.get_path_lookup()
                error_msg = dropBoxErrorMsg(lookUpError._tag)
                raise ContentDoesNotExistsError(error_msg)
            raise e

    def get_created_time(self, name):
        """
        Return the creation time (as a datetime) of the file specified by name.
        The datetime will be timezone-aware if USE_TZ=True.
        """
        full_name = self._full_path(name)
        try:
            created_at = self.dbx.files_get_metadata(full_name).client_modified
            return created_at
        except ApiError as e:
            err = e.error
            if err.is_path_lookup():
                lookUpError = err.get_path_lookup()
                error_msg = dropBoxErrorMsg(lookUpError._tag)
                raise ContentDoesNotExistsError(error_msg)
            raise e

    def get_modified_time(self, name):
        """
        Return the last modified time (as a datetime) of the file specified by
        name. The datetime will be timezone-aware if USE_TZ=True.
        """
        full_name = self._full_path(name)
        try:
            last_modified = self.dbx.files_get_metadata(full_name).server_modified
            return last_modified
        except ApiError as e:
            err = e.error
            if err.is_path_lookup():
                lookUpError = err.get_path_lookup()
                error_msg = dropBoxErrorMsg(lookUpError._tag)
                raise ContentDoesNotExistsError(error_msg)
            raise e
                
    def _chunked_upload(self, content, dest_path):
        upload_session = self.dbx.files_upload_session_start(content.read(self.CHUNK_SIZE))
        cursor = UploadSessionCursor(session_id=upload_session.session_id, offset=content.tell())
        commit = CommitInfo(path=dest_path, mode=WriteMode(self.write_mode))
        while content.tell() < content.size:
            if (content.size - content.tell()) <= self.CHUNK_SIZE:
                self.dbx.files_upload_session_finish(content.read(self.CHUNK_SIZE), cursor, commit)
            else:
                self.dbx.files_upload_session_append_v2(content.read(self.CHUNK_SIZE), cursor)
                cursor.offset = content.tell()


from pathlib import Path
from decouple import config, Csv
import environ

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

env_file = BASE_DIR / '.env'
env = environ.Env()

DEBUG = True

APPWRITE_API_KEY = config('APPWRITE_API_KEY')
APPWRITE_PROJECT_ID = config('APPWRITE_PROJECT_ID')
APPWRITE_BUCKET_ID = config('APPWRITE_BUCKET_ID')

CLOUD_STORAGE_CREATE_NEW_IF_SAME_CONTENT = False
DROPBOX_OAUTH2_ACCESS_TOKEN = config('DROPBOX_OAUTH2_ACCESS_TOKEN')
DROPBOX_OAUTH2_REFRESH_TOKEN = config('DROPBOX_OAUTH2_REFRESH_TOKEN')
DROPBOX_APP_KEY = config('DROPBOX_APP_KEY')
DROPBOX_APP_SECRET = config('DROPBOX_APP_SECRET')
DROPBOX_ROOT_PATH = "/Apps/my_personal_website_data/media"
DROPBOX_PERMANENT_LINK = True
OVERWRITE_FILE =True

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:'
    }
}
SECRET_KEY = 'justfortest'



# from cloud_storages.backends.appwrite import *
# from django.test import *


# class AppWriteStorageTest(TestCase):
#     @classmethod
#     def setUpTestData(cls):
#         # print("setUpTestData: Run once to set up non-modified data for all class methods.")
#         pass
    
#     def setUp(self, *args):
#         # print("setUp: Run once for every test method to setup clean data.")
#         self.storage = AppWriteStorage()
#         self.file_name = "Samiddha_Photo.JPG"

#     def tearDown(self, *args):
#         # Clean up run after every test method.
#         pass

#     def test_open(self, *args):
#         self.storage.open(name=self.file_name)

#     # def test_save(self, *args):
#     #     test_content = ContentFile(content="Test File")
#     #     self.storage.save(name=self.file_name, content=test_content)
    
#     # def test_delete(self, *args):
#     #     self.storage.delete(name=self.file_name)


import requests
import secrets
import string
import traceback
import re
from io import BytesIO

from django.core.files.base import ContentFile
from django.core.files.storage import Storage
from django.core.files import File
from django.utils.deconstruct import deconstructible
from django.utils.deconstruct import deconstructible

from appwrite.role import Role
from appwrite.permission import Permission
from appwrite.client import Client
from appwrite.input_file import InputFile
from appwrite.services.storage import Storage as appwrite_storage
from appwrite.exception import AppwriteException

from cloud_storages.utils import *


@deconstructible
class AppWriteStorage(Storage):
    CHUNK_SIZE = 4 * 1024 * 1024
    MAX_FILE_NAME_LENGTH = 30
    def __init__(self):
        self.OVERWRITE_FILE = setting('OVERWRITE_FILE', False)
        self.CLOUD_STORAGE_CREATE_NEW_IF_SAME_CONTENT = setting('CLOUD_STORAGE_CREATE_NEW_IF_SAME_CONTENT', True)
        self.APPWRITE_API_KEY = setting('APPWRITE_API_KEY')
        self.APPWRITE_PROJECT_ID = setting('APPWRITE_PROJECT_ID')
        self.APPWRITE_BUCKET_ID = setting('APPWRITE_BUCKET_ID')
        self.APPWRITE_API_ENDPOINT = setting('APPWRITE_API_ENDPOINT', "https://cloud.appwrite.io/v1")
        self.MEDIA_URL = setting('MEDIA_URL')
        
        self.client = Client()
        (self.client
        .set_endpoint(self.APPWRITE_API_ENDPOINT) # Your API Endpoint
        .set_project(self.APPWRITE_PROJECT_ID) # Your project ID
        .set_key(self.APPWRITE_API_KEY) # Your secret API key
        )
        self.storage = appwrite_storage(self.client)

    def open(self, name, mode="rb"):
        """Retrieve the specified file from storage."""
        return self._open(name, mode)
    def _open(self, name, mode='rb'):
        folder, filename = self.extract_folder_and_filename(name)
        try:
            response = self.storage.get_file_view(folder, filename)
            file_content = BytesIO(response)
            response_file = File(file_content, name=name)
            return response_file
        except AppwriteException as e:
            if str(e) == "Storage bucket with the requested ID could not be found.":
                error_msg = f"The folder does not exists on the cloud storage.\nFolder Name: {folder}."
                raise ContentDoesNotExistsError(error_msg)
            elif str(e) == "The requested file could not be found.":
                error_msg = f"The file does not exists on the cloud storage.\nFile Name: {filename}."
                raise ContentDoesNotExistsError(error_msg)
            else:
                raise e
    
    def save(self, name, content, max_length=None):
        """
        Save new content to the file specified by name. The content should be
        a proper File object or any Python file-like object, ready to be read
        from the beginning.
        """
        folder, org_filename = self.extract_folder_and_filename(name)
        content = File(content, org_filename)
        # if not hasattr(content, "chunks"):
        #     content = File(content, org_filename)
        folder = self.create_folder(folder)
        path = self.get_available_name(name=name, content=content, max_length=max_length)
        if path[1] is None:
            self._save(path[0], content)
        return path[0]
    def _save(self, path, content):
        folder, filename = self.extract_folder_and_filename(path)
        content.open()
        the_file = InputFile.from_bytes(bytes=content.read(), filename=content.name)
        result = self.storage.create_file(bucket_id=folder, file_id=filename, file=the_file)
        content.close()
        return path
    
    def _id_validation(self, name):
        name = re.sub(r"^\W+|^_+", "", name) # Remove special characters at beginning
        name = re.sub(r"[^a-zA-Z0-9-._]*", "", name)
        return name

    def extract_folder_and_filename(self, path):
        path = str(path).replace("\\", "/")
        folders = path.split("/")
        if len(folders) == 0:  # file.txt
            last_folder = self.APPWRITE_BUCKET_ID
            file_name = path
        elif len(folders) == 1:  # /file.txt
            last_folder = self.APPWRITE_BUCKET_ID
            file_name = folders[0]
        else: # folder/folder2/file.txt
            file_name = folders.pop()  # eliminate the last item i.e. file name
            last_folder = folders.pop()
        last_folder = last_folder if len(last_folder) <= self.MAX_FILE_NAME_LENGTH else last_folder[0:30]
        last_folder = self._id_validation(last_folder)
        # file_name = self.get_valid_name(file_name)
        return (last_folder, file_name)
    
    def create_folder(self, folder):
        try:
            result = self.storage.get_bucket(folder)
        except AppwriteException as e:
            if str(e) == "Storage bucket with the requested ID could not be found.":
                result = self.storage.create_bucket(bucket_id=folder, name=folder, permissions=[Permission.read(Role.any())], file_security=False, enabled=True, encryption=False, antivirus=False)
            else:
                raise e
        return folder

    def get_available_name(self, name, content, max_length=None):
        """
        Return a filename that's free on the target storage system and
        available for new content to be written to.
        """
        formatted_name = self.get_valid_name(name)
        new_name = formatted_name
        if self.OVERWRITE_FILE:
            try:
                self.delete(new_name)
            except ContentDoesNotExistsError:
                pass
        else:
            index = 0
            while(1):
                index += 1
                if self.exists(new_name): # check file existence with file name
                    if not self.CLOUD_STORAGE_CREATE_NEW_IF_SAME_CONTENT:
                        # check file existence with file's contents
                        remote_file = self.open(new_name)
                        remote_file_data = remote_file.read()
                        content_data = content.read()
                        if remote_file_data == content_data:
                            return (new_name, 'Exists')
                    new_name = self.get_alternative_name(formatted_name, index=index)
                    continue
                break
        return (new_name, None)
    
    def generate_filename(self, filename):
        """
        Validate the filename by calling get_valid_name() and return a filename
        to be passed to the save() method.
        """
        name = self.get_valid_name(filename)
        return name

    def get_valid_name(self, name):
        """
        Return a filename, based on the provided filename, that's suitable for
        use in the target storage system.
        """
        folder, filename = self.extract_folder_and_filename(name)
        filename = self._id_validation(filename)
        res = filename.rsplit('.', 1)  # Split on last occurrence of delimiter
        if len(res[0])>self.MAX_FILE_NAME_LENGTH:
            err_msg = f"File name's length must be less than or equal to {self.MAX_FILE_NAME_LENGTH}."
            raise FileNameLengthError(max_length=self.MAX_FILE_NAME_LENGTH, message=err_msg)
        elif len(res[0]) < 1:
            err_msg = f"File name's length is too small. Avoid using special characters in file name, and file name should not start with underscore."
            raise FileNameLengthError(message=err_msg)
        return f"{folder}/{filename}"
        # return {'file_id': filename, 'file_name': filename, 'folder_id': folder, 'folder_name': folder}

    def get_alternative_name(self, file_root, file_ext=None, index=0):
        """
        Return an alternative filename if one exists to the filename.
        """
        folder, filename = self.extract_folder_and_filename(file_root)
        res = filename.rsplit('.', 1)  # Split on last occurrence of delimiter
        file_name = f"{res[0]}-{index}"
        file_ext = res[1]
        updated_name =  f"{file_name}.{file_ext}"
        return f"{folder}/{updated_name}"
        # return {'file_id': updated_name, 'file_name': updated_name, 'folder_id': file_root['folder_id'], 'folder_name': file_root['folder_name']}

    def delete(self, name):
        """
        Delete the specified file from the storage system.
        """
        folder, filename = self.extract_folder_and_filename(name)
        try:
            result = self.storage.delete_file(folder, filename)
        except AppwriteException as e:
            if str(e) == "Storage bucket with the requested ID could not be found.":
                error_msg = f"The folder does not exists on the cloud storage.\nFolder Name: {folder}."
                raise ContentDoesNotExistsError(error_msg)
            elif str(e) == "The requested file could not be found.":
                error_msg = f"The file does not exists on the cloud storage.\nFile Name: {filename}."
                raise ContentDoesNotExistsError(error_msg)
            else:
                raise e

    def exists(self, name):
        """
        Return True if a file referenced by the given name already exists in the
        storage system, or False if the name is available for a new file.
        """
        folder, filename = self.extract_folder_and_filename(name)
        try:
            result = self.storage.get_file(folder, filename)
            return True
        except AppwriteException as e:
            if str(e) == "The requested file could not be found." or str(e) == "Storage bucket with the requested ID could not be found.":
                return False
            else:
                raise e
        
    def listdir(self, path):
        """
        List the contents of the specified path. Return a 2-tuple of lists:
        the first item being directories, the second item being files.
        """
        pass
    
    def size(self, name):
        """
        Return the total size, in bytes, of the file specified by name.
        """
        folder, filename = self.extract_folder_and_filename(name)
        try:
            result = self.storage.get_file(folder, filename)
            return result.sizeOriginal
        except AppwriteException as e:
            if str(e) == "Storage bucket with the requested ID could not be found.":
                error_msg = f"The folder does not exists on the cloud storage.\nFolder Name: {folder}."
                raise ContentDoesNotExistsError(error_msg)
            elif str(e) == "The requested file could not be found.":
                error_msg = f"The file does not exists on the cloud storage.\nFile Name: {filename}."
                raise ContentDoesNotExistsError(error_msg)
            else:
                raise e
    
    def url(self, name, permanent_link=None):
        """
        Return an absolute URL where the file's contents can be accessed directly by a web browser.
        """
        folder, filename = self.extract_folder_and_filename(name)
        file_url = f"{self.APPWRITE_API_ENDPOINT}/storage/buckets/{folder}/files/{filename}/view?project={self.APPWRITE_PROJECT_ID}"
        return file_url
    
    def get_accessed_time(self, name):
        """
        Return the last accessed time (as a datetime) of the file specified by name.
        The datetime will be timezone-aware if USE_TZ=True.
        """
        folder, filename = self.extract_folder_and_filename(name)
        try:
            result = self.storage.get_file(folder, filename)
            return result.updatedAt
        except AppwriteException as e:
            if str(e) == "Storage bucket with the requested ID could not be found.":
                error_msg = f"The folder does not exists on the cloud storage.\nFolder Name: {folder}."
                raise ContentDoesNotExistsError(error_msg)
            elif str(e) == "The requested file could not be found.":
                error_msg = f"The file does not exists on the cloud storage.\nFile Name: {filename}."
                raise ContentDoesNotExistsError(error_msg)
            else:
                raise e

    def get_created_time(self, name):
        """
        Return the creation time (as a datetime) of the file specified by name.
        The datetime will be timezone-aware if USE_TZ=True.
        """
        folder, filename = self.extract_folder_and_filename(name)
        try:
            result = self.storage.get_file(folder, filename)
            return result.createdAt
        except AppwriteException as e:
            if str(e) == "Storage bucket with the requested ID could not be found.":
                error_msg = f"The folder does not exists on the cloud storage.\nFolder Name: {folder}."
                raise ContentDoesNotExistsError(error_msg)
            elif str(e) == "The requested file could not be found.":
                error_msg = f"The file does not exists on the cloud storage.\nFile Name: {filename}."
                raise ContentDoesNotExistsError(error_msg)
            else:
                raise e
    
    def get_modified_time(self, name):
        """
        Return the last modified time (as a datetime) of the file specified by
        name. The datetime will be timezone-aware if USE_TZ=True.
        """
        folder, filename = self.extract_folder_and_filename(name)
        try:
            result = self.storage.get_file(folder, filename)
            return result.updatedAt
        except AppwriteException as e:
            if str(e) == "Storage bucket with the requested ID could not be found.":
                error_msg = f"The folder does not exists on the cloud storage.\nFolder Name: {folder}."
                raise ContentDoesNotExistsError(error_msg)
            elif str(e) == "The requested file could not be found.":
                error_msg = f"The file does not exists on the cloud storage.\nFile Name: {filename}."
                raise ContentDoesNotExistsError(error_msg)
            else:
                raise e